/*
 Expandable KVM
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/07/2014
 Last Modified: 6/09/2014

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define SW1 PA0
#define SW2 PA1
#define LED1 PA2
#define LED2 PA3
#define QUAD_SW_ENABLE PA4
#define QUAD_SW_IN PA5
#define ANA_SW1_SELECT PA6
#define ANA_SW2_SELECT PA7
#define HIGH_Z PB2

// Sleep - powerdown
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(uint8_t ii) {
	if (ii > 9) ii = 9;
	uint8_t bb = ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCSR = bb;
	WDTCSR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCSR |= (1<<WDCE) | (1<<WDE);
	WDTCSR = 0;
	sei(); // Turn interrupts back on
}

// Watchdog sleep function - combines all the above functions into one
void watchdog_sleep(int timeout) { 
	setup_watchdog(timeout);
	system_sleep();
	turnOffwatchdog();
}

// Setup the AVR
void setup(void) {
	// Set outputs
	DDRA |= ((1<<LED1) | (1<<LED2) | (1<<QUAD_SW_ENABLE) | (1<<QUAD_SW_IN) | (1<<ANA_SW1_SELECT) | (1<<ANA_SW2_SELECT));
	
	// High-Z the quad analog switch and select 100K resistor on the 2x single analog switch
	PORTA |= (1<<QUAD_SW_ENABLE);
	PORTA |= (1<<ANA_SW1_SELECT) | (1<<ANA_SW2_SELECT);
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE0); // Enable pin change interrupt
	sbi(GIMSK,PCIE1);
	sbi(PCMSK0,PCINT0); // Button 1 and 2
	sbi(PCMSK0,PCINT1);
	sbi(PCMSK1,PCINT10); // HighZ reading pin
	
	// Turn on interrupt
	sei();
}
